// TrendView.cpp : implementation file
//
// Trend Display ver 1.0
//
// Written by Majid Shahabfar (shahabfar@safineh.net)
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included.
//
// This file is provided with no expressed or implied warranty.
// The author accepts no responsibility if it causes any damage to your
// computer.
//
// Please use and enjoy and let me know of any bugs/improvements 
// that you have found/implemented and I will fix/incorporate them into this
// code. 
//
// An email letting me know how you are using it would be nice as well.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TrendTest.h"
#include "TrendView.h"
#include "TrendDoc.h"

// CTrendView

IMPLEMENT_DYNCREATE(CTrendView, CFormView)

CTrendView::CTrendView()
	: CFormView(CTrendView::IDD)
{
	m_TempValue = 30;
	m_scanTime = 500; // milli second
	srand((unsigned)time(NULL));
}

CTrendView::~CTrendView()
{
    delete m_pTrendList;
}

void CTrendView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CTrendView, CFormView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_TIMER()
END_MESSAGE_MAP()


// CTrendView diagnostics

#ifdef _DEBUG
void CTrendView::AssertValid() const
{
	CFormView::AssertValid();
}

void CTrendView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif //_DEBUG

// CTrendView message handlers

void CTrendView::OnDraw(CDC* pDC)
{
	pDC->SetBkMode(TRANSPARENT);

	GetClientRect(m_rect);

	m_trendWndRect.left   = (LONG)(m_rect.Width()*.01);
	m_trendWndRect.top    = (LONG)(m_rect.Height()*.05);
	m_trendWndRect.right  = (LONG)(m_rect.Width()*.99);
	m_trendWndRect.bottom = (LONG)(m_rect.Height()*.65);

	CBrush br(RGB(0,0,0));
	CBrush* pbrOld = pDC->SelectObject(&br);
	pDC->Rectangle(m_trendWndRect);
	pDC->SelectObject(pbrOld);

    DrawGridLines(pDC);
	DrawRulers(pDC,m_rect.Width());
    DrawScale(pDC);
	DrawTime(pDC);
	/* ~~~~~~~~~Ruler~~~~~~~~~*/
	CPen pen1(PS_SOLID,1,RGB(255,255,175));
	CPen* pOldPen = pDC->SelectObject(&pen1);
	pDC->MoveTo(m_trendRect.left+m_timeStep,m_trendRect.bottom);
	pDC->LineTo(m_trendRect.left+m_timeStep,m_trendRect.top);
	if(m_trendRect.left+m_timeStep >= m_trendRect.right)
		m_timeStep = 0;
	pDC->SelectObject(pOldPen);
	/*~~~~~~~~~~~~~~~ ~~~~~~~~*/
}

void CTrendView::DrawTime(CDC* pDC)
{
	CString str;
	CFont font;
	int size = m_trendRect.Width()/8;
    font.CreatePointFont(size, _T ("Times"));
	CFont* pOldFont = pDC->SelectObject(&font);
	pDC->SetTextColor(RGB(0,0,0));
    str.Format(_T("%02d:%02d:%2d"),m_initialTime.GetHour(),m_initialTime.GetMinute(),m_initialTime.GetSecond());
    pDC->DrawText(str,CRect(m_trendRect.left-50,m_trendRect.bottom+5,m_trendRect.left,m_trendRect.bottom+20),DT_LEFT);

	int t = (m_trendRect.Width()*m_scanTime)/1000;
	m_tempTime = CTimeSpan( 0, 0, 0, t ); 
	m_finalTime = m_initialTime + m_tempTime;
    str.Format(_T("%02d:%02d:%2d"),m_finalTime.GetHour(),m_finalTime.GetMinute(),m_finalTime.GetSecond());
	pDC->DrawText(str,CRect(m_trendRect.right,m_trendRect.bottom+5,m_trendRect.right+50,m_trendRect.bottom+20),DT_LEFT);

	int verticalStep = m_trendRect.Width()/4;
	t = (m_trendRect.Width()*m_scanTime)/4000;
	m_tempTime = CTimeSpan( 0, 0, 0, t ); 
    m_stepTime[0] = m_initialTime + m_tempTime;

	t = (m_trendRect.Width()*m_scanTime)/2000;
	m_tempTime = CTimeSpan( 0, 0, 0, t ); 
    m_stepTime[1] = m_initialTime + m_tempTime;

	t = (m_trendRect.Width()*m_scanTime)/1333;
	m_tempTime = CTimeSpan( 0, 0, 0, t ); 
    m_stepTime[2] = m_initialTime + m_tempTime;

	for(BYTE i=0 ; i<3 ; i++)
	{
		str.Format(_T("%02d:%02d:%2d"),m_stepTime[i].GetHour(),m_stepTime[i].GetMinute(),m_stepTime[i].GetSecond());
		pDC->DrawText(str,CRect(m_trendRect.left+verticalStep,m_trendRect.bottom+5,m_trendRect.left+50+verticalStep,m_trendRect.bottom+20),DT_LEFT);
		verticalStep += m_trendRect.Width()/4;
	}

	CString strScopeTime = "Scope Time: ";
	m_tempTime = m_finalTime - m_initialTime;
	str.Format(_T("%02d:%02d:%2d"),m_tempTime.GetHours(),m_tempTime.GetMinutes(),m_tempTime.GetSeconds());
	strScopeTime += str;
	pDC->DrawText(strScopeTime,CRect(m_trendRect.CenterPoint().x-65,
		m_trendRect.CenterPoint().y-m_trendRect.Height()/2-20,m_trendRect.CenterPoint().x+65,
		m_trendRect.CenterPoint().y-m_trendRect.Height()/2-5),DT_LEFT);
	pDC->SelectObject(pOldFont);
}

void CTrendView::DrawRulers(CDC* pDC,int width)
{
	CPen pen1(PS_SOLID,1,RGB(0,80,0));
	m_OldPen = pDC->SelectObject(&pen1);

	pDC->MoveTo(int(width*.095),m_trendRect.top);
	pDC->LineTo(int(width*.095),int(m_trendRect.bottom-1.5));
	pDC->MoveTo(int(width*.904),m_trendRect.top);
	pDC->LineTo(int(width*.904),int(m_trendRect.bottom-1.5));

	float step = 0;
	for(float i=1; i<=9 ; i++)
	{
		if(i==9)
		{
			pDC->MoveTo(int(width*.095),int(m_trendRect.bottom-1.5));
			pDC->LineTo(int(width*.095-10),int(m_trendRect.bottom-1.5));
			pDC->MoveTo(int(width*.904),int(m_trendRect.bottom-1.5));
			pDC->LineTo(int(width*.904+10),int(m_trendRect.bottom-1.5));
			break;
		}
		if((i/2) == int(i/2))
		{
			pDC->MoveTo(int(width*.095),int(m_trendRect.top+step));
			pDC->LineTo(int(width*.095-5),int(m_trendRect.top+step));
			pDC->MoveTo(int(width*.904),int(m_trendRect.top+step));
			pDC->LineTo(int(width*.904+5),int(m_trendRect.top+step));
		}
		else
		{
			pDC->MoveTo(int(width*.095),int(m_trendRect.top+step));
			pDC->LineTo(int(width*.095-10),int(m_trendRect.top+step));
			pDC->MoveTo(int(width*.904),int(m_trendRect.top+step));
			pDC->LineTo(int(width*.904+10),int(m_trendRect.top+step));
		}
		step += m_trendRect.Height()/8;
	}

	pDC->SelectObject(m_OldPen);
}

void CTrendView::DrawGridLines(CDC* pDC)
{
	CPen pen1(PS_DOT,1,RGB(0,80,0));
	m_OldPen = pDC->SelectObject(&pen1);
	float verticalStep = 0;
	for(BYTE k=0 ; k<4 ; k++)
	{
		if(k==0 || k==4)
		{
    		verticalStep += m_trendRect.Width()/4;
			continue;
		}
		pDC->MoveTo(int(m_trendRect.left+verticalStep),m_trendRect.top);
		pDC->LineTo(int(m_trendRect.left+verticalStep),m_trendRect.bottom+5);
		verticalStep += m_trendRect.Width()/4;
	}

	float step = 0;
	for(float i=1; i<=9 ; i++)
	{
		if(i==1 || i==9)
		{
            step += m_trendRect.Height()/8;
			continue;
		}
		pDC->MoveTo(m_trendRect.left,int(m_trendRect.top+step));
		pDC->LineTo(m_trendRect.right,int(m_trendRect.top+step));
		step += m_trendRect.Height()/8;
	}
	pDC->SelectObject(m_OldPen);
}

void CTrendView::DrawScale(CDC* pDC)
{
	CString str;
	CFont font;
	int size = m_trendRect.Width()/8;
    font.CreatePointFont(size, _T ("Times"));
	CFont* pOldFont = pDC->SelectObject(&font);
	int yStep = m_trendRect.Height()/4; 	
	int yDistance = 0;
	for(BYTE i=0 ; i<m_pDoc->m_noOfPoint ; i++)
	{
		if(i==3)
			yDistance = 0;
		float step = (m_pDoc->m_pPoint[i].Max - m_pDoc->m_pPoint[i].Min)/4;
		pDC->SetTextColor(m_pDoc->m_pPoint[i].Color);
		str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Max);
		switch(m_pDoc->m_noOfPoint)
		{
		case 1:
			pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.top+1,m_trendRect.left-15,m_trendWndRect.top+15),DT_LEFT);
			str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
			pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-15,m_trendRect.left-15,m_trendWndRect.bottom-2),DT_LEFT);
			for(BYTE j=1 ; j<=3 ; j++)
			{
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-15-(yStep*j),m_trendRect.left-15,m_trendWndRect.bottom-2-(yStep*j)),DT_LEFT);
			}
			break;
		case 2:
			if(i==0)
			{
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.top+1,m_trendRect.left-15,m_trendWndRect.top+15),DT_LEFT);
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-15,m_trendRect.left-15,m_trendWndRect.bottom-2),DT_LEFT);
				for(BYTE j=1 ; j<=3 ; j++)
				{
					str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
					pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-15-(yStep*j),m_trendRect.left-15,m_trendWndRect.bottom-2-(yStep*j)),DT_LEFT);
				}
			}
			else
			{
				pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.top+1,m_trendWndRect.right-10,m_trendWndRect.top+15),DT_LEFT);
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
				pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.bottom-15,m_trendWndRect.right-10,m_trendWndRect.bottom-2),DT_LEFT);
				for(BYTE j=1 ; j<=3 ; j++)
				{
					str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
					pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.bottom-15-(yStep*j),m_trendWndRect.right-10,m_trendWndRect.bottom-2-(yStep*j)),DT_LEFT);
				}
			}
			break;
		case 3:
			pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.top+1+yDistance,m_trendRect.left-15,m_trendWndRect.top+15+yDistance),DT_LEFT);
			str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
			pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-15-yDistance,m_trendRect.left-15,m_trendWndRect.bottom-2-yDistance),DT_LEFT);
			for(BYTE j=1 ; j<=3 ; j++)
			{
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-20-(yStep*j)+yDistance,m_trendRect.left-15,m_trendWndRect.bottom-7-(yStep*j)+yDistance),DT_LEFT);
			}
			break;
		case 4:
			if(i<3)
			{
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.top+1+yDistance,m_trendRect.left-15,m_trendWndRect.top+15+yDistance),DT_LEFT);
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-15-yDistance,m_trendRect.left-15,m_trendWndRect.bottom-2-yDistance),DT_LEFT);
				for(BYTE j=1 ; j<=3 ; j++)
				{
					str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
					pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-20-(yStep*j)+yDistance,m_trendRect.left-15,m_trendWndRect.bottom-7-(yStep*j)+yDistance),DT_LEFT);
				}
			}
			else
			{
				pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.top+1,m_trendWndRect.right-10,m_trendWndRect.top+15),DT_LEFT);
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
				pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.bottom-15,m_trendWndRect.right-10,m_trendWndRect.bottom-2),DT_LEFT);
				for(BYTE j=1 ; j<=3 ; j++)
				{
					str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
					pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.bottom-15-(yStep*j),m_trendWndRect.right-10,m_trendWndRect.bottom-2-(yStep*j)),DT_LEFT);
    			}
			}
			break;
		case 5:
			if(i<3)
			{
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.top+1+yDistance,m_trendRect.left-15,m_trendWndRect.top+15+yDistance),DT_LEFT);
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-15-yDistance,m_trendRect.left-15,m_trendWndRect.bottom-2-yDistance),DT_LEFT);
				for(BYTE j=1 ; j<=3 ; j++)
				{
					str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
					pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-20-(yStep*j)+yDistance,m_trendRect.left-15,m_trendWndRect.bottom-7-(yStep*j)+yDistance),DT_LEFT);
				}
			}
			else
			{
				pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.top+1+yDistance,m_trendWndRect.right-10,m_trendWndRect.top+15+yDistance),DT_LEFT);
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
				pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.bottom-15-yDistance,m_trendWndRect.right-10,m_trendWndRect.bottom-2-yDistance),DT_LEFT);
				for(BYTE j=1 ; j<=3 ; j++)
				{
					str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
					pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.bottom-20-(yStep*j)+yDistance,m_trendWndRect.right-10,m_trendWndRect.bottom-7-(yStep*j)+yDistance),DT_LEFT);
    			}
			}
			break;
		case 6:
			if(i<3)
			{
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.top+1+yDistance,m_trendRect.left-15,m_trendWndRect.top+15+yDistance),DT_LEFT);
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
				pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-15-yDistance,m_trendRect.left-15,m_trendWndRect.bottom-2-yDistance),DT_LEFT);
				for(BYTE j=1 ; j<=3 ; j++)
				{
					str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
					pDC->DrawText(str,CRect(m_trendWndRect.left+10,m_trendWndRect.bottom-20-(yStep*j)+yDistance,m_trendRect.left-15,m_trendWndRect.bottom-7-(yStep*j)+yDistance),DT_LEFT);
				}
			}
			else
			{
				pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.top+1+yDistance,m_trendWndRect.right-10,m_trendWndRect.top+15+yDistance),DT_LEFT);
				str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min);
				pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.bottom-15-yDistance,m_trendWndRect.right-10,m_trendWndRect.bottom-2-yDistance),DT_LEFT);
				for(BYTE j=1 ; j<=3 ; j++)
				{
					str.Format(m_pDoc->m_pPoint[i].FOR,m_pDoc->m_pPoint[i].Min+(step*j));
					pDC->DrawText(str,CRect(m_trendRect.right+20,m_trendWndRect.bottom-20-(yStep*j)+yDistance,m_trendWndRect.right-10,m_trendWndRect.bottom-7-(yStep*j)+yDistance),DT_LEFT);
    			}
			}
			break;
		}
		yDistance += 15;
	}
	pDC->SelectObject(pOldFont);
}

int CTrendView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFormView::OnCreate(lpCreateStruct) == -1)
		return -1;

	GetClientRect(m_rect);
	m_trendRect.left   = LONG(m_rect.Width()*.1);
	m_trendRect.top    = LONG(m_rect.Height()*.05);
	m_trendRect.right  = LONG(m_rect.Width()*.9);
	m_trendRect.bottom = LONG(m_rect.Height()*.65);

	m_trendWndRect.left   = LONG(m_rect.Width()*.01);
	m_trendWndRect.top    = LONG(m_rect.Height()*.05);
	m_trendWndRect.right  = LONG(m_rect.Width()*.99);
	m_trendWndRect.bottom = LONG(m_rect.Height()*.65);

	m_pTrendList = new CListCtrl;
	
	m_trendListRect.left   = LONG(0.04*m_trendRect.Width());
	m_trendListRect.top    = LONG(0.8*m_trendRect.Height());
	m_trendListRect.right  = LONG(0.97*m_trendRect.Width());
	m_trendListRect.bottom = LONG(0.96*m_trendRect.Height());
	
	m_pTrendList->Create(LVS_OWNERDRAWFIXED | LVS_REPORT | LVS_ALIGNLEFT | LVS_SHOWSELALWAYS | LVS_SINGLESEL,
		m_trendListRect,this,IDC_LIST_TRENDLIST);

	return 0;
}

void CTrendView::OnSize(UINT nType, int cx, int cy)
{
	CFormView::OnSize(nType, cx, cy);

	CTrendDoc* pDoc = (CTrendDoc*)GetDocument();

	GetClientRect(&m_rect);

	m_trendRect.left   = LONG(m_rect.Width()*.1);
	m_trendRect.top    = LONG(m_rect.Height()*.05);
	m_trendRect.right  = LONG(m_rect.Width()*.9);
	m_trendRect.bottom = LONG(m_rect.Height()*.65);

	m_trendListRect.left   = LONG(0.04*m_rect.Width());
	m_trendListRect.top    = LONG(0.8*m_rect.Height());
	m_trendListRect.right  = LONG(0.97*m_rect.Width());
	m_trendListRect.bottom = LONG(0.96*m_rect.Height());

	m_pTrendList->SetWindowPos(&wndTop,m_trendListRect.left,m_trendListRect.top,
		m_trendListRect.right  - m_trendListRect.left,
		m_trendListRect.bottom - m_trendListRect.top,SWP_SHOWWINDOW | SWP_FRAMECHANGED);

	if(m_timeStep > m_trendRect.Width())
	{
		for(BYTE i=0 ; i<pDoc->m_noOfPoint ; i++)
    	{
            pDoc->m_pPoint[i].BufferedValue.RemoveAll();
		}
		m_timeStep = 0;
		m_initialTime = CTime::GetCurrentTime();
		Invalidate();
		_heapmin();
	}
}

void CTrendView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();

	m_pTrendList->Arrange(LVA_SNAPTOGRID);
	
	m_pTrendList->InsertColumn( 0,"Color",LVCFMT_LEFT,m_pTrendList->GetStringWidth("Color")+25,0);
    m_pTrendList->InsertColumn( 1,"Name",LVCFMT_LEFT,m_pTrendList->GetStringWidth("Name ")+105,1);
    m_pTrendList->InsertColumn( 2,"Description",LVCFMT_LEFT,m_pTrendList->GetStringWidth("Description")+125,2);
    m_pTrendList->InsertColumn( 3,"Min    ",LVCFMT_LEFT,m_pTrendList->GetStringWidth("Min    ")+25,5);
    m_pTrendList->InsertColumn( 4,"Max    ",LVCFMT_LEFT,m_pTrendList->GetStringWidth("Max    ")+25,6);
    m_pTrendList->InsertColumn( 5,"Current Value",LVCFMT_LEFT,m_pTrendList->GetStringWidth("Current Val")+25,7);
	m_pTrendList->InsertColumn( 6,"Unit",LVCFMT_LEFT,m_pTrendList->GetStringWidth("Unit")+25,6);

	m_pDoc = (CTrendDoc*)GetDocument();
	m_timeStep = 0;
	m_initialTime = CTime::GetCurrentTime();
	StartTimer();
	FillTrendListCtrl();
}

void CTrendView::OnTimer(UINT nIDEvent)
{
	static count = 0;
	count++;
	if(count==15)
	{
		m_TempValue = float((unsigned int)(((LONG)rand() * (30)) / (RAND_MAX + 1)));
		count = 0;
	}
	for(BYTE i=0 ; i<m_pDoc->m_noOfPoint ; i++)
	{
		m_pDoc->m_pPoint[i].Val = m_TempValue;
		if(m_TempValue > m_pDoc->m_pPoint[i].Max)
			m_pDoc->m_pPoint[i].Val = m_pDoc->m_pPoint[i].Max;
		if(m_TempValue < m_pDoc->m_pPoint[i].Min)
			m_pDoc->m_pPoint[i].Val = m_pDoc->m_pPoint[i].Min;
    	m_pDoc->m_pPoint[i].BufferedValue.Add(m_pDoc->m_pPoint[i].Val);
		FillList(i,m_pDoc->m_pPoint[i].Val);
	} 
	m_timeStep++;
	if(m_trendRect.left+m_timeStep >= m_trendRect.right)
	{
		for(BYTE i=0 ; i<m_pDoc->m_noOfPoint ; i++)
    	{
            m_pDoc->m_pPoint[i].BufferedValue.RemoveAll();
		}
		m_timeStep = 0;
		m_initialTime = CTime::GetCurrentTime();
		Invalidate();
		_heapmin();
	}
    DrawTrend();
	CFormView::OnTimer(nIDEvent);
}

void CTrendView::StartTimer()
{
	SetTimer(TRENDTIMER,m_scanTime,NULL);
}

void CTrendView::StopTimer()
{
	KillTimer(TRENDTIMER);
}
void CTrendView::DrawTrend()
{
	m_pDC = GetDC();
	for(BYTE j=0 ; j<m_pDoc->m_noOfPoint ; j++)
	{
		CPen pen(PS_SOLID,1,m_pDoc->m_pPoint[j].Color);
		m_OldPen = m_pDC->SelectObject(&pen);
		for(int i=1 ; i<m_pDoc->m_pPoint[j].BufferedValue.GetSize() ; i++)
		{
			m_portion = (m_pDoc->m_pPoint[j].BufferedValue[i-1]-m_pDoc->m_pPoint[j].Min)/m_pDoc->m_pPoint[j].Range;
			m_pDC->MoveTo(m_trendRect.left+i-1,int(m_trendRect.bottom-m_portion*m_trendRect.Height()));
			m_portion = (m_pDoc->m_pPoint[j].BufferedValue[i]-m_pDoc->m_pPoint[j].Min)/m_pDoc->m_pPoint[j].Range;
			m_pDC->LineTo(m_trendRect.left+i,int(m_trendRect.bottom-m_portion*m_trendRect.Height()));
		}
		m_pDC->SelectObject(m_OldPen);
	}
	InvalidateRect(CRect(m_trendRect.left+m_timeStep-1,m_trendRect.top,m_trendRect.left+m_timeStep+1,m_trendRect.bottom));
	ReleaseDC(m_pDC);
}

void CTrendView::FillTrendListCtrl()
{
	CTrendDoc* pDoc = (CTrendDoc*)GetDocument();

    m_pTrendList->DeleteAllItems();
	LV_ITEM       lvi;
	char buf[60];

	lvi.mask       = LVIF_TEXT; 
	lvi.pszText    = LPSTR_TEXTCALLBACK;
	lvi.cchTextMax = 5;
	lvi.iSubItem   = 0;

	for(int i=0 ; i<pDoc->m_noOfPoint ; i++)
	{
		lvi.iItem = i;
		lvi.iImage= i;
		m_pTrendList->InsertItem(&lvi);
		m_pTrendList->SetItemText(i,0,"");
		m_pTrendList->SetItemText(i,1,pDoc->m_pPoint[i].Name);
		m_pTrendList->SetItemText(i,2,pDoc->m_pPoint[i].Dsc);
		sprintf(buf,"%.3f",(float)pDoc->m_pPoint[i].Min);
		m_pTrendList->SetItemText(i,3,buf);
		sprintf(buf,"%.3f",(float)pDoc->m_pPoint[i].Max);
		m_pTrendList->SetItemText(i,4,buf);
		strcpy(buf,"    ");
		m_pTrendList->SetItemText(i,5,buf);
		m_pTrendList->SetItemText(i,6,pDoc->m_pPoint[i].UNI);
	}
}

void CTrendView::FillList(BYTE index,float Value)
{
	LV_ITEM       lvi;
	char buf[60];

	lvi.mask       = LVIF_TEXT; 
	lvi.pszText    = LPSTR_TEXTCALLBACK;
	lvi.cchTextMax = 5;
	lvi.iSubItem   = 0;
	
	sprintf(buf,"%.3f",Value);
	m_pTrendList->SetItemText(index,5,buf);
}

// Owner draw list control
LRESULT CTrendView::MyDrawItem(LPDRAWITEMSTRUCT lp)
{
	CTrendDoc* pDoc = (CTrendDoc*)GetDocument();
	RECT 	rect;
	char    buf[60];
	int     center;
	int     offset = 0;
	HPEN    NewPen;
	HGDIOBJ OldPen;

	center = (lp->rcItem.bottom - lp->rcItem.top)  / 2;
	
	rect.left  =lp->rcItem.left;
	rect.top   =lp->rcItem.top;
	rect.right =lp->rcItem.left + m_pTrendList->GetColumnWidth(0);
	rect.right+=m_pTrendList->GetColumnWidth(1);
	rect.bottom=lp->rcItem.bottom;
	
	if (lp->itemState & ODS_SELECTED)
	{
		CBrush     FillColor(RGB(115,185,255));
		FillRect(lp->hDC, &rect,(HBRUSH)FillColor);
	}
	else
	{
        FillRect(lp->hDC, &rect,(HBRUSH) GetStockObject(WHITE_BRUSH));
	}

	SetTextColor(lp->hDC,RGB(0,0,0));
    NewPen = CreatePen(PS_SOLID,
		               2,
		               pDoc->m_pPoint[lp->itemID].Color);
    OldPen=SelectObject(lp->hDC,(HGDIOBJ)NewPen);
	rect.left  =lp->rcItem.left + 2;
	rect.top   =lp->rcItem.top  +2;
	rect.right =lp->rcItem.left + m_pTrendList->GetColumnWidth(0) - 2;
	rect.bottom=lp->rcItem.bottom - 2;
	
	FillRect(lp->hDC, &rect,CBrush(RGB(0,0,0)));
	MoveToEx(lp->hDC,lp->rcItem.left + 2 ,lp->rcItem.top + center,NULL);
	LineTo(lp->hDC,lp->rcItem.left + m_pTrendList->GetColumnWidth(0) - 3,lp->rcItem.top + center);
	
	offset +=m_pTrendList->GetColumnWidth(0);
	rect= lp->rcItem;
	rect.left = lp->rcItem.left + offset + 2;
	rect.right =lp->rcItem.left + offset + m_pTrendList->GetColumnWidth(1) - 2;
	m_pTrendList->GetItemText(lp->itemID,1,buf,60);
    DrawText(lp->hDC,buf,int(strlen(buf)),&rect,DT_LEFT);   
	
	SetTextColor(lp->hDC,RGB(0,0,0));
	offset += m_pTrendList->GetColumnWidth(1);
	rect= lp->rcItem;
	rect.left = lp->rcItem.left + offset + 2;
	rect.right =lp->rcItem.left + offset + m_pTrendList->GetColumnWidth(2) - 2;
	m_pTrendList->GetItemText(lp->itemID,2,buf,60);
    DrawText(lp->hDC,buf,int(strlen(buf)),&rect,DT_LEFT);   
	
	SetTextColor(lp->hDC,RGB(0,0,0));
	
	offset += m_pTrendList->GetColumnWidth(2);
	rect= lp->rcItem;
	rect.left = lp->rcItem.left + offset + 2;
	rect.right =lp->rcItem.left + offset + m_pTrendList->GetColumnWidth(3) - 2;
	m_pTrendList->GetItemText(lp->itemID,3,buf,60);
    DrawText(lp->hDC,buf,int(strlen(buf)),&rect,DT_LEFT);   
	
	offset += m_pTrendList->GetColumnWidth(3);
	rect= lp->rcItem;
	rect.left = lp->rcItem.left + offset + 2;
	rect.right =lp->rcItem.left + offset + m_pTrendList->GetColumnWidth(4) - 2;
	m_pTrendList->GetItemText(lp->itemID,4,buf,60);
    DrawText(lp->hDC,buf,int(strlen(buf)),&rect,DT_LEFT);   
	
	offset += m_pTrendList->GetColumnWidth(4);
	rect= lp->rcItem;
	rect.left = lp->rcItem.left + offset + 2;
	rect.right =lp->rcItem.left + offset + m_pTrendList->GetColumnWidth(5) - 2;
	m_pTrendList->GetItemText(lp->itemID,5,buf,60);
    DrawText(lp->hDC,buf,int(strlen(buf)),&rect,DT_LEFT);
	
	offset += m_pTrendList->GetColumnWidth(5);
	rect= lp->rcItem;
	rect.left = lp->rcItem.left + offset + 2;
	rect.right =lp->rcItem.left + offset + m_pTrendList->GetColumnWidth(6) - 2;
	m_pTrendList->GetItemText(lp->itemID,6,buf,60);
    DrawText(lp->hDC,buf,int(strlen(buf)),&rect,DT_LEFT);   

    offset += m_pTrendList->GetColumnWidth(6);
	rect= lp->rcItem;
	rect.left = lp->rcItem.left + offset + 2;
	rect.right =lp->rcItem.left + offset + m_pTrendList->GetColumnWidth(7) - 2;
	m_pTrendList->GetItemText(lp->itemID,7,buf,60);
    DrawText(lp->hDC,buf,int(strlen(buf)),&rect,DT_LEFT);   

	DeleteObject((HGDIOBJ)NewPen);
    SelectObject(lp->hDC,OldPen);
	
	return TRUE;
}

LRESULT CTrendView::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	LPDRAWITEMSTRUCT lp;
	lp= (LPDRAWITEMSTRUCT)lParam;
	switch(message )
	{
	case WM_DRAWITEM:
		{
			if(wParam == IDC_LIST_TRENDLIST)
				return MyDrawItem(lp);
		}
		break;
	default: ;
		
	}
	return CView::WindowProc(message, wParam, lParam);
}
